'-------------------------------------------------------------
'					(c) 2001 BiPOM Electronics
'-------------------------------------------------------------
' This module is presented here only to serve as a sample for
' 8051 Basic  program development on the MINI-MAX/51-C system.
' BiPOM Electronics does not guarantee its functionality
' with other micro-controller systems.
' Please e-mail any questions and suggestions to 
' bipom@compuserve.com
'
' Module: X10.BAS
'
' Revision: 1.01
'
' Description: Example demonstrates how to control
' X10 devices using X10.lib.
' This code was tested with MINI-MAX/51-C board 
' (AT89S53, 22.1108 MHz crystal). 
'-------------------------------------------------------------------------

$LIB "X10.lib"
' detects the power line frequensy
$external X10Auto
' Assembles and transmits X10 block
$external X10TX
' Receives and disassembles X10 block
$external X10RX
' Provides 2 power line cycles gap
$external X10GAP

$BAUD = 9600										
$CRYSTAL = 22110800	

'Lines, aligned to X10
X10ZC   alias  P3.2    'Zero crossing
X10RXD  alias  P3.4    'Received data
X10TXD  alias  P3.5    'Transmit data 

DIM House as Byte, Key as Byte  'House Code, Key Code
DIM T_OUT as bit                'Receive time out
DIM S as String*16              'Command line buffer
DIM a as BYTE, c as byte        'General purpose variables

Declare Sub X10TRANS 
Declare Sub X10RCV 
Declare Sub SVAL

XAU01:
    call X10Auto        'power line frequensy auto detect
$asm
    mov a,PCON
    mov c,acc.2
    mov {T_OUT},c
$end asm  
    If T_OUT = 1 then 
    Print "No X10 device"
    input a
    goto XAU01
    end if 
       
DO   
    INPUT ">",S         'input command line
    S = UCASE(S)

' get 1-st symbol for analysis 
    a = VARPTR(S)
$asm
    mov r0,{a}
    mov a,@r0
    mov {a},a
$end asm
    
    if a = 72 then   'H - house code

'Convert House (A...P) into House Code
    a = VARPTR(S)
$asm
    mov r0,{a}
    inc r0
    mov a,@r0
    clr c
    subb a,#63
    movc a,@A+pc
    ajmp xh01
db  6,14,2,10,1,9,5,13
db  7,15,3,11,0,8,4,12
xh01:
    mov {a},a
$end asm

    House = a        'Save house Code
    goto m001
    end if
    

    if a = 85 then   'U - Unit Code

'Convert Unit into Unit Code    
    a = VARPTR(S)
m002:
    call SVAL        '= "VAL",ignorring 1-st symbol  
$asm    
    mov a,{a} 
    add a,#1
    movc a,@A+pc
    ajmp xh02
db  12,28,4,20,2,18,10,26
db  14,30,6,22,0,16,8,24
xh02:
    mov {a},a
$end asm
    Key = a

'Transmit twice House Code + Unit Code
    call X10TRANS

'  Were there more Units entered?
'  subroutine SVAL saves pointer
'  to the next value in var. c
'  if last symbol was ','
'  else c=0

    if c = 0 then
    goto m001          'No more Units
    end if
    a=c
    goto m002          'Next Unit
    end if 


    if a = 75 then    'K - Key code 

    a = VARPTR(S)
m004:
'Convert Unit into Unit Code
    call SVAL        '= "VAL",ignorring 1-st symbol 
    Key = a

'Transmit twice House Code + Key Code
    call X10TRANS

'  Were there more Keys entered?
'  subroutine SVAL saves pointer
'  to the next value in var. c
'  if last symbol was ','
'  else c=0


    if c = 0 then
    goto m001         'No more Keys
    end if
    a=c
    goto m004         'Next Key

    end if

    if a = 82 then    'R - enter receive mode
    goto XRCV
    end if

    if a=63 then      '? - help
    print "  Command      Number"
    print "All untts OFF    1"
    print "All units ON     3"
    print "ON          5"
    print "OFF         7"
    print "Dim         9"
    print "Bright      11"
    print "All lights OFF   13"
    print "Extended code    15"
    print "Hail request     17"
    print "Hail acknowledge 19"
    print "Pre-set Dim      21" 
    print "Extended data    23"
    print "Status is ON     25"
    print "Status is OFF    27"
    print "Status request   29"
    end if
    
m001:
    loop
    
    
'    Converts a string representation
'    of a number into a number, 
'    ignoring 1-st symbol
 
Sub SVAL
$asm
     mov r0,{a}
     mov r1,r0
     mov r2,#0
xc02:
     inc r0
     mov a,@r0
     clr c
     subb a,#48
     jc xc01
     mov a,@r0
     clr c
     subb a,#58
     jnc xc01
     ajmp xc02 

xc01:
     mov {c},r0 
     mov a,@r0
     mov r3,a
     dec r0
     mov a,r1
     cjne a,r0,xc07
     ljmp xc06      
xc07:
     mov a,@r0
     anl a,#15
     add a,r2
     mov r2,a
     
     dec r0
     mov a,r1
     cjne a,r0,xc08
     ljmp xc06      
xc08:
     mov a,@r0
     anl a,#15
     mov b,#10
     mul ab
     add a,r2
     mov r2,a    
xc06:
     mov {a},r2
     cjne r3,#44,xc09
     ajmp xc10
xc09:
     mov {c},#0
xc10:
     nop     
$end asm
end sub



' Receiving X10 blocks and shows received
' House Codes and Key codes.

XRCV:
       call X10RCV         'receive X10 block

       If T_OUT =1  then   'There was Time-Out

$asm                       'Check was there a key pressed 
      mov {a},#0
      jnb SCON.0,xr01
      inc {a}
      clr SCON.0
xr01:
      nop
$end asm 
     
      if a=1 then       'a key was pressed - 
      goto m001         'abort reception
      end if
      
      goto XRCV         'There was not a key pressed
      end if             

'There was not Time-Out,
'show received House Code and Key/Unit Code
      print "H=";House;" K="; Key 

'Check was there a key pressed 
$asm
      mov {a},#0
      jnb SCON.0,xr02
      inc {a}
      clr SCON.0
xr02:
      nop
$end asm 
     
      if a=1 then       'yes - abort reception
      goto m001
      end if

      goto XRCV         'no, receive next 


' Assembles and transmits standard X10 block    
Sub X10TRANS
$asm
	mov a,{House}
	mov b,{Key}
	lcall X10TX
	lcall X10GAP
	mov a,{House}
	mov b,{Key}
	lcall X10TX
	lcall X10GAP	
$end asm
End Sub

' Receives and disassembles X10 block
Sub X10RCV
$asm
	lcall X10RX
    mov {House},a
	mov {Key},b
    mov {T_OUT},c 
$end asm
End Sub
    

	
  